#!/usr/bin/perl

# This program (Perl script) is called by CIM to obtain the health data
# for XTier. This program reads and parses XTier statistics files, and
# returns a list of key=value pairs to CIM via STDOUT.
#
# novell-xsrvd.xml describes the key=value pairs that are returned.
#
# Files used in this script:
#  xserv.conf  - the xsrvd configuration file (read only)
#                /etc/opt/novell/xtier/xsrvd/xsrvd.conf
   $xconf = "/etc/opt/novell/xtier/xsrvd/xsrvd.conf";

#  envvars     - the xsrvd environment variables file (read only)
#                /etc/opt/novell/xtier/xsrvd/envvars
   $envvars = "/etc/opt/novell/xtier/xsrvd/envvars";

#  ncpl-stats  - the xsrvd statistics files (read only)
#                Suffix with "n.xml" where n is the daemon number starting at 0.
#                /var/opt/novell/xtier/xsrvd/srv-ncpl-stats-
   $nstats = "/var/opt/novell/xtier/xsrvd/srv-ncpl-stats-";

#  xtier-stats - the XTier statistics file returned from
#                http://127.0.0.1/oneNet/xtier-stats
#                This can be a temoprary file as it is first written, then read.
#                /var/opt/novell/xtier/xsrvd/xtier-stats
   $xstats = "/var/opt/novell/xtier/xsrvd/xtier-stats";


sub keyvalue()
{
   # This subroutine reads a key/value text file searching for
   # keys and getting their associated values (string or numeric).
   #
   # Input file has lines like:
   #    # comment line
   #    key=value
   #
   # $_[0]    = filename to read
   # %keyhash = hash of keys to look for and their values
   # $debug   = 0 is off, 1 for debug printout

   my @keylist = keys %keyhash;
   my $key;

   # Read in the entire file
   my $ercd = 0;
   $ercd = open(FILE, "<$_[0]") or $ercd = 0;
   if (!$ercd)
   {
      if ($debug)
      { print "db-Cannot open file $_[0]\n"; }
      return;
   }
   my @filecontent = <FILE>;
   close(FILE);

   # Check each line of the file
   foreach my $line(@filecontent)
   {
      # Ignore all comment lines starting with #
      if ($line =~ m/^#/)
      { next; }

      # Check for the key names
      foreach $key(@keylist)
      {
         # Look for key name, 0 or more whitespace,
         #    '=', 0 or more whitespace, any chars, whitespace(nl)
         if ($line =~ m/$key\s*=\s*(.+)\s/)
         {
            if ($debug)
            {
               print "\ndb-Line matched: $line";
               print "db-envvar matched: $key\n";
               print "db-Value matched: $1\n";
            }
            $keyhash{$key} = $1;
         }
      }
   }
   return;
}


sub xmlvalue()
{
   # This subroutine reads an XML text file searching for tags and
   # accumulates or stores their values.
   #
   # If parameter $_[1] is 0, this subroutine stores the values.
   # The subroutine exits with the last value processed in the case
   # of duplicate tags.
   #
   # If parameter %_[1] is 1, this subroutine expects all numeric
   # values and sums them in the case of duplicate tags. This option
   # will not work for string values.
   #
   # XML file has line break before relevant values:
   #    <tag>
   #    value</tag>
   #
   # $_[0]    = filename to read
   # $_[1]    = 0 then store values, = 1 then sum values
   # %keyhash = hash of XML tags to look for and their accumulated
   #            numeric values
   # $debug   = 0 is off, 1 for debug printout

   my @keylist = keys %keyhash;
   my $key;

   # Read in the entire file
   #
   my $ercd = 0;
   $ercd = open(FILE, "<$_[0]") or $ercd = 0;
   if (!$ercd)
   {
      if ($debug)
      { print "db-Cannot open file $_[0]\n"; }
      return;
   }
   my @xtierstats = <FILE>;
   close(FILE);

   my $xtag;
   my $tagFound = 0;

   # Check each line of the file
   foreach my $line(@xtierstats)
   {
      if ($tagFound)
      {
         $line =~ m|</|;
         if ($_[1])
         { $keyhash{$xtag} += $`; }
         else
         { $keyhash{$xtag} = $`; }
         $tagFound = 0;
         if ($debug)
         {
            print "db-Value matched: $`\n";
            print "db-Total Value now: $keyhash{$xtag}\n";
         }
         next;
      }

      # Check for the key names
      foreach $key(@keylist)
      {
         if ($line =~ m/<$key>/)
         {
            if ($debug)
            { print "\ndb-tag matched: $key\n"; }
            $xtag = $key;
            $tagFound = 1;
            last;
         }
      }
   }
   return;
}


###
# Main program starts here
#

$debug = 0;


###
# Parse xsrvd.conf
#
%keyhash = ("NumberOfServers" => 10);

&keyvalue($xconf);

# Output last occurrence to CIM
$key = "NumberOfServers";
$nos = $keyhash{$key};
print "$key=$nos\n";


###
# Parse envvars
#
%keyhash = ("XTIER_CODE_PAGE"                  => "ANSI_X3.64-1968",
            "XSRVD_NO_AUTORESTART_AFTER_CRASH" => 0,
            "XSRVD_COREDUMPS_WANTED"           => 0);

&keyvalue($envvars);

# Output last occurrence of each variable to CIM
foreach $key(keys %keyhash)
{ print "$key=$keyhash{$key}\n"; }


###
# Parse ncpl-stats
#
for ($daemon = 0; $daemon < $nos; $daemon++)
{
   # Reinit to 0 each time
   %keyhash = ("paged-memory"     => 0,
               "nonpaged-memory"  => 0,
               "read-write-locks" => 0,
               "work-items"       => 0);

   &xmlvalue($nstats.$daemon.".xml", 1);     # sum option

   # Output to CIM
   #    Combine paged and non-paged memory
   $mem = $keyhash{"paged-memory"} + $keyhash{"nonpaged-memory"};
   print "memory-allocated$daemon=$mem\n";
   $key = "read-write-locks";
   print $key."$daemon=$keyhash{$key}\n";
   $key = "work-items";
   print $key."$daemon=$keyhash{$key}\n";
}


###
# Parse xtier-stats
#

%keyhash = ("server-up"                     => "forever",
            "total-requests"                => 0,
            "bytes-read"                    => 0,
            "bytes-written"                 => 0,
            "outstanding-requests"          => 0,
            "sessions"                      => 0,
            "max-concurrent-sessions"       => 0,
            "sessions-created"              => 0,
            "authentication-failovers"      => 0,
            "login-failures"                => 0,
            "memory-allocation-failures"    => 0,
            "io-buffer-allocation-failures" => 0);

# Shell to w3m to get XML file from XTier
@args = ("bash", "-c", "w3m http://127.0.0.1/oneNet/xtier-stats >$xstats");
$ret = system(@args);

if ($ret == 0)
{
   #Success

   &xmlvalue($xstats, 0);     # store option

   # Output last occurrence of each variable to CIM
   foreach $key(keys %keyhash)
   { print "$key=$keyhash{$key}\n"; }
}
elsif ($debug)
{ print "db-w3m error = $ret\n"; }

# xsrvd-health.pl
